import logging

log = logging.getLogger('upgrader')

class IVersionRepository(Interface):

    def get_version(self, name, default=None):
        pass

    def save_version(self, name, version):
        pass


class FileRepository(IVersionRepository):

    def __init__(self, filename):
        import ConfigParser

        config = ConfigParser.RawConfigParser()
        self.inifile = config.read(filename)

    def get_version(self, name, default=None):
        self.inifile.getInt('Versions', name)

    def save_version(self, name, version):
        self.inifile.setInt('Versions', name, version)



class DeclarativeVersioning(object):

    def __init__(self, declarative_base, repository):
        """Initialize versioning on :declarative_base:"""
        self.base = declarative_base
        self.reqistry = self.base._decl_class_registry
        self.repository = repository

        for name, instance in self.registry.iteritems():
            if not hasattr(instance, '__version__'):
                log.error('Declarative object %s doesn\'t have __version__' % name)
                continue
            log.info('%s, __version__ = %s' % (name, instance.__version__))

    def upgrade_all(self):
        pass